/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2004/08/18 09:32:04 $ $Revision: 1.12 $ */
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <stdio.h>
#include "global.h"
#include "anim.h"
#include "menu.h"
#include "XPM/fcont.xpm"
#include "XPM/forward.xpm"
#include "XPM/bcont.xpm"
#include "XPM/backward.xpm"
#include "XPM/marker.xpm"
#include "XPM/space.xpm"
#include "XPM/tobeg.xpm"
#include "XPM/rew.xpm"
#include "XPM/play.xpm"
#include "XPM/bstep.xpm"
#include "XPM/pause.xpm"
#include "XPM/stop.xpm"
#include "XPM/ff.xpm"
#include "XPM/toend.xpm"
#include "XPM/redraw.xpm"

/* Prototypes */
void Menu_Run(GtkWidget *,gpointer);
gint AnimateBody(gpointer);
gint Animation(gpointer);
gint AnimframeSlider_Changed(GtkWidget *, GtkWidget *);
gint AnimframeValue_Changed(GtkWidget *, GtkObject *);
void Toggle_Adjustanimframe(GtkToggleButton *);
int ilog(double);
void FrameOff(GtkWidget *,gpointer);
void Replot_Objects(int);
void Reshape_Anim(void);
void Marker_Move(GtkWidget *,GdkEvent *);

static char **playdirXPM[4] = {	XPMfcont,
				XPMforward,
  				XPMbcont,
  				XPMbackward };

void Pause_Signal(GtkWidget *widget, gpointer data)
{
  int a;

  a=(int)data;
  if (a) {
    animation.playdir=(animation.playdir+2)%4;
  }
  if (anim.running) {
    Menu_Run(widget,(gpointer)1);
  } else {
    AnimateBody((gpointer)gl.glarea);
  }
  if (a) {
    animation.playdir=(animation.playdir+2)%4;
  }
}

void BegEnd_Signal(GtkWidget *widget, gpointer data)
{
  long a;

  a=(long)data;
  if (a) {	/* end */
    animation.framepc=(animation.maxframe/anim.frameskip)*anim.frameskip;
  } else {	/* beginning */
    animation.framepc=0;
  }
  GTK_ADJUSTMENT(animation.playposslideradj)->value=animation.framepc;
  gtk_signal_emit_by_name(GTK_OBJECT (animation.playposslideradj),
      "value_changed");
}

void FF_Signal(GtkWidget *widget, gpointer data)
{
  long a;
  int sgn;

  a=(long)data;
  if (a&2) {	/* released */
    anim.ff=1;
  } else {	/* pressed */
    sgn=(1-(a&1)*2);
    anim.ff=animation.ffval*sgn;
  }
}

void Set_PlayDir(GtkWidget *widget, gpointer data)
{
  GtkStyle  *style;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  animation.playdir=(int) data;
  style= gtk_widget_get_style(animation.playframe);
  pixmap = gdk_pixmap_create_from_xpm_d (animation.playframe->window, &mask,
		&style->bg[GTK_STATE_NORMAL],playdirXPM[animation.playdir]);
  gtk_pixmap_set(GTK_PIXMAP(animation.playdirw),pixmap,mask);
}

void ChangeQuadRes(GtkWidget *widget)
{
  gl.quad.st=atoi(gtk_entry_get_text(GTK_ENTRY(widget)));
  gl.quad.sl=2*gl.quad.st;
  if (!anim.running) Replot_Objects(0);
}

/* rotate play direction */
void RotatePlayDir_Signal(GtkWidget *widget, gpointer data)
{
  int a;

  a=(animation.playdir+1)%4;
  Set_PlayDir(widget,(gpointer) a);
}

void Toggle(GtkWidget *widget, gpointer data)
{
  if (*(int *)data>1) *(int *)data=1;
  *(int *)data^=1;
}

/* The value of frameskip was set by user */
void Frameskip_Set(GtkWidget *widget, gpointer data)
{
  anim.frameskip=atoi(gtk_entry_get_text(GTK_ENTRY(widget)));
  GTK_ADJUSTMENT(animation.playposslideradj)->value=(gfloat)animation.framepc;
  GTK_ADJUSTMENT(animation.playposslideradj)->upper=(gfloat)animation.maxframe;
  gtk_signal_emit_by_name(GTK_OBJECT (animation.playposslideradj), "changed");
  gtk_signal_emit_by_name(GTK_OBJECT (animation.playposslideradj),
      							"value_changed");
}

/* adds a pixmap (filename) button to hbox on the window/frame window
 * and returns the widget of the button */
GtkWidget *Pixmap_Button_New_To_Hbox(GtkWidget *window,GtkWidget *hbox,
				GtkWidget **pixmapw,char **xpmname)
{
  GtkStyle  *style;	/* style for pixmap background */
  GdkPixmap *pixmap;	/* pixmap for pixmap buttons */
  GdkBitmap *mask;	/* pixmap mask */
  GtkWidget *box;	/* pixmap box widget */
  GtkWidget *button;	/* button widget, to be returned */

  style= gtk_widget_get_style(window);
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &mask,
		     &style->bg[GTK_STATE_NORMAL],xpmname);
  *pixmapw=gtk_pixmap_new(pixmap,mask);
  gdk_pixmap_unref(pixmap);
  box = gtk_vbox_new(FALSE,0);
  button=gtk_button_new();
  gtk_box_pack_start(GTK_BOX(box),*pixmapw,FALSE,FALSE,3);
  gtk_container_add(GTK_CONTAINER(button),box);
  gtk_box_pack_start(GTK_BOX(hbox),button,FALSE,FALSE,0);
  gtk_widget_show(*pixmapw);
  gtk_widget_show(box);
  gtk_widget_show(button);
  return(button);
}

gint PlayPosSlider_Changed(GtkWidget *widget, GtkWidget *playposbox)
{
  int l;
  char *s;

  l=ilog(GTK_ADJUSTMENT(widget)->value);
  s=(char *) calloc(l+1,sizeof(char));
  sprintf(s,"%d",(int)(GTK_ADJUSTMENT(widget)->value));
  gtk_entry_set_text(GTK_ENTRY(playposbox),s);
  free(s);
  animation.framepc=(int)(GTK_ADJUSTMENT(widget)->value);
  Animation(gl.glarea);
  return 1;
}

gint PlayPosBox_Changed(GtkWidget *widget, GtkObject *playposslideradj)
{
  int v;

  v=atoi(gtk_entry_get_text(GTK_ENTRY(widget)));
  if (v>GTK_ADJUSTMENT(playposslideradj)->upper) {
    v=(int) GTK_ADJUSTMENT(playposslideradj)->upper;
  }
  GTK_ADJUSTMENT(playposslideradj)->value=v;
  gtk_signal_emit_by_name (GTK_OBJECT (playposslideradj), "value_changed");
  return(1);
}

void Create_AnimFrame_Frame(GtkWidget *frame)
{
  GtkWidget *animframevframebox;/* v packing box for animframe frame */
  GtkWidget *animframehframebox;/* h packing box for slider etc. */
  GtkWidget *h2box,*h3box;	/* toggle buttons */
  GtkWidget *h4box;		/* toggle buttons */
  GtkWidget *animframeslider;	/* animframe frequency slider */
  GtkObject *animframeslideradj;/* adjustment parameters for animframe slider */
  GtkWidget *animframevalue;	/* animframe frequency value, text box */
  GtkWidget *adjustanimframe;	/* toggle butt whether animframe is adjusted */
  GtkWidget *detachcontrol;	/* toggle for detach retation from timeout */
  GtkWidget *quadstw;		/* widget to print quad resolution */
  GtkWidget *quadslidectrl;	/* toggle quad slide number adjustment */
  GtkWidget *donotplot;		/* toggle texture usage */
  GtkWidget *calconread;	/* toggle calculate only on read */
  GtkWidget *savepng;		/* toggle always save png*/
  GtkWidget *label;		/* a label */
  GtkWidget *off;		/* switch frame off */
  char s[3];

  animframevframebox = gtk_vbox_new(FALSE, 3);
  animframehframebox = gtk_hbox_new(FALSE, 0);
  h2box = gtk_hbox_new(FALSE, 0);
  h3box = gtk_hbox_new(FALSE, 0);
  h4box = gtk_hbox_new(FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), animframevframebox);
  gtk_box_pack_start(GTK_BOX(animframevframebox), animframehframebox,
      	TRUE, TRUE, 1);

  /* horisontal slider */
  animframeslideradj=gtk_adjustment_new((double) anim.framedt, 1.0,
      				1000.0, 1.0, 10.0, 1.0);
  animframeslider=gtk_hscale_new(GTK_ADJUSTMENT(animframeslideradj));
  gtk_widget_set_usize(animframeslider,200,0);
  gtk_scale_set_digits(GTK_SCALE(animframeslider),0);
  gtk_range_set_update_policy(GTK_RANGE(animframeslider),
      GTK_UPDATE_DELAYED);
  gtk_scale_set_draw_value(GTK_SCALE(animframeslider),FALSE);
  gtk_box_pack_start(GTK_BOX(animframehframebox), animframeslider,
      TRUE, TRUE, 5);

  /* entry box for the value of the horizontal slider */
  animframevalue=gtk_entry_new_with_max_length(
      ilog(GTK_ADJUSTMENT(animframeslideradj)->upper));
  gtk_widget_set_usize(animframevalue,60,0);
  gtk_entry_set_text(GTK_ENTRY(animframevalue), "0");
  gtk_box_pack_start(GTK_BOX(animframehframebox), animframevalue, 
      	FALSE, FALSE, 5);
  /* Connect these boxes */
  gtk_signal_connect (GTK_OBJECT (animframeslideradj), "value_changed",
	    GTK_SIGNAL_FUNC (AnimframeSlider_Changed), animframevalue);
  gtk_signal_connect (GTK_OBJECT (animframevalue), "activate",
	    GTK_SIGNAL_FUNC (AnimframeValue_Changed), animframeslideradj);
  gtk_box_pack_start(GTK_BOX(animframevframebox), h2box,TRUE,TRUE,1);
  gtk_box_pack_start(GTK_BOX(animframevframebox), h3box,TRUE,TRUE,1);
  gtk_box_pack_start(GTK_BOX(animframevframebox), h4box,TRUE,TRUE,1);
  adjustanimframe=gtk_check_button_new_with_label("Adjust Frame freq.");
  if (anim.adjustframedt) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (adjustanimframe), TRUE);
  } else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (adjustanimframe), FALSE);
  }
  gtk_box_pack_start(GTK_BOX(h2box), adjustanimframe,FALSE,FALSE,0);
  quadslidectrl=gtk_check_button_new_with_label(
      "Adjust quad. obj. resolution");
  gtk_box_pack_start(GTK_BOX(h2box), quadslidectrl,FALSE,FALSE,0);
  gl.quadstw=quadstw=gtk_entry_new_with_max_length(3);
  gtk_widget_set_usize(quadstw,25,0);
  sprintf(s,"%d",min(99,gl.quad.st));
  gtk_entry_set_text(GTK_ENTRY(quadstw), s);
  gtk_box_pack_start(GTK_BOX(h2box), quadstw, 
      	FALSE, FALSE, 5);
  if (anim.adjustquadres) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (quadslidectrl), TRUE);
  } else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (quadslidectrl), FALSE);
  }
  detachcontrol=gtk_check_button_new_with_label(
      				"Detach rotation from Frame freq.");
  gtk_box_pack_start(GTK_BOX(h3box), detachcontrol,FALSE,FALSE,0);
  donotplot=gtk_check_button_new_with_label("Replot on button");
  gtk_box_pack_start(GTK_BOX(h3box), donotplot,FALSE,FALSE,0);
  anim.detach=0;
  anim.animframeslideradj=animframeslideradj;
  anim.animframevalue=animframevalue;
  label=gtk_label_new("Always: ");
  gtk_box_pack_start(GTK_BOX(h4box), label,FALSE,FALSE,0);
  calconread=gtk_check_button_new_with_label("Calculate constraint");
  gtk_box_pack_start(GTK_BOX(h4box), calconread,FALSE,FALSE,0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (calconread), TRUE);
  savepng=gtk_check_button_new_with_label("Save png");
  gtk_box_pack_start(GTK_BOX(h4box), savepng,FALSE,FALSE,0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (savepng), FALSE);
  off=gtk_check_button_new_with_label("Off");
  gtk_box_pack_end(GTK_BOX(h4box),off,FALSE,FALSE,0);
  gtk_widget_show(quadstw);
  gtk_widget_show(label);
  gtk_widget_show(savepng);
  gtk_widget_show(calconread);
  gtk_widget_show(donotplot);
  gtk_widget_show(quadslidectrl);
  gtk_widget_show(detachcontrol);
  gtk_widget_show(off);
  gtk_widget_show(h2box);
  gtk_widget_show(h3box);
  gtk_widget_show(h4box);

  gtk_signal_connect (GTK_OBJECT (adjustanimframe), "map_event",
                        GTK_SIGNAL_FUNC(Toggle_Adjustanimframe), NULL);
  gtk_signal_connect (GTK_OBJECT (adjustanimframe), "toggled",
                        GTK_SIGNAL_FUNC(Toggle_Adjustanimframe), NULL);
  /* set text box according to slider value */
  gtk_signal_emit_by_name (GTK_OBJECT (anim.animframeslideradj),
    							"value_changed");
  gtk_signal_connect (GTK_OBJECT (detachcontrol), "toggled",
	    GTK_SIGNAL_FUNC (Toggle), &anim.detach);
  gtk_signal_connect (GTK_OBJECT (quadslidectrl), "toggled",
	    GTK_SIGNAL_FUNC (Toggle), (gpointer) &anim.adjustquadres);
  gtk_signal_connect (GTK_OBJECT (quadstw), "activate",
	    GTK_SIGNAL_FUNC (ChangeQuadRes), NULL);
  gtk_signal_connect (GTK_OBJECT (calconread), "toggled",
	    GTK_SIGNAL_FUNC (Toggle), &anim.constrreread);
  gtk_signal_connect (GTK_OBJECT (savepng), "toggled",
	    GTK_SIGNAL_FUNC (Toggle), &anim.save_png);
  gtk_signal_connect (GTK_OBJECT (donotplot), "toggled",
	    GTK_SIGNAL_FUNC (Toggle), &animation.donotplot);
  gtk_signal_connect (GTK_OBJECT (off), "toggled",
	    GTK_SIGNAL_FUNC (FrameOff), (gpointer)frame);
  gtk_widget_show(adjustanimframe);
  gtk_widget_show(animframevalue);
  gtk_widget_show(animframeslider);
  gtk_widget_show(animframehframebox);
  gtk_widget_show(animframevframebox);

}

void Create_Animation_Buttons(GtkWidget *frame)
{
  GtkWidget *mvbox;		/* Main vbox */
  GtkWidget *playhbox;		/* play buttons */
  GtkWidget *playvbox;		/* markers */
  GtkWidget *play;		/* play button */
  GtkWidget *stop;		/* stop button */
  GtkWidget *bstep;		/* backstep button */
  GtkWidget *pause;		/* pause button */
  GtkWidget *ff;		/* ff button */
  GtkWidget *rew;		/* rew button */
  GtkWidget *tobeg;		/* to beginning button */
  GtkWidget *toend;		/* to end button */
  GtkWidget *redraw;		/* redraw button */
  GtkWidget *dummy;		/* dummy widget */
  GtkWidget *playdir;		/* play direction changer */
  GtkWidget *sliderhbox;	/* hbox for slider and value */
  GtkWidget *playposslider;	/* slider to play position */
  GtkObject *playposslideradj;	/* adjustment for the latter */
  GtkWidget *playposbox;	/* box of play position */
  GtkWidget *frameskip;		/* textbox of frameskip value */
  GtkWidget *hboxmarker;	/* hbox for markers */
  GtkWidget *markereventbox;	/* event box for markers */
  GtkWidget *marker1,*marker2;	/* markers for slider */
  GtkWidget *space1,*space2;	/* space for markers */
  GtkStyle  *style;		/* style for pixmap background */
  GdkPixmap *pixmap;		/* pixmap for pixmap buttons */
  GdkBitmap *mask;		/* pixmap mask */
  char s[6];			/* string for frameskip */

  mvbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame),mvbox);
  playhbox = gtk_hbox_new(FALSE, 0);
  sliderhbox = gtk_hbox_new(FALSE, 0);
  playvbox = gtk_vbox_new(FALSE, 0);
  markereventbox = gtk_event_box_new();
  hboxmarker = gtk_hbox_new(FALSE, 0);
  /* slider and value box */
  gtk_box_pack_start(GTK_BOX(mvbox),sliderhbox,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(sliderhbox),playvbox,FALSE,FALSE,5);
  animation.playposslideradj=playposslideradj=gtk_adjustment_new(0.0, 0.0,
      		(gfloat) animation.maxframe+1.0,1.0, 10.0, 1.0);
  animation.playposslider=
    playposslider=gtk_hscale_new(GTK_ADJUSTMENT(playposslideradj));
  gtk_widget_set_usize(playposslider,290,0);
  gtk_scale_set_digits(GTK_SCALE(playposslider),0);
  gtk_range_set_update_policy(GTK_RANGE(playposslider),
      GTK_UPDATE_DELAYED);
  gtk_scale_set_draw_value(GTK_SCALE(playposslider),FALSE);
  gtk_box_pack_start(GTK_BOX(playvbox), playposslider, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(playvbox), markereventbox,FALSE,FALSE,0);
  gtk_container_add(GTK_CONTAINER(markereventbox),hboxmarker);

  /* Pixmap for marker */
  style= gtk_widget_get_style(frame);
  pixmap = gdk_pixmap_create_from_xpm_d (frame->window, &mask,
		     &style->bg[GTK_STATE_NORMAL],XPMmarker);
  marker1=gtk_pixmap_new(pixmap,mask);
  marker2=gtk_pixmap_new(pixmap,mask);
  gdk_pixmap_unref(pixmap);
  style= gtk_widget_get_style(frame);
  pixmap = gdk_pixmap_create_from_xpm_d (frame->window, &mask,
		     &style->bg[GTK_STATE_NORMAL],XPMspace);
  animation.space1=space1=gtk_pixmap_new(pixmap,mask);
  animation.space2=space2=gtk_pixmap_new(pixmap,mask);
  gdk_pixmap_unref(pixmap);
  gtk_widget_set_usize(space1,1,0);
  gtk_box_pack_start(GTK_BOX(hboxmarker), space1, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hboxmarker), marker1, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hboxmarker), space2, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hboxmarker), marker2, FALSE, FALSE, 0);
  dummy=gtk_label_new("Ctrls");
  gtk_widget_set_usize(dummy,30,17);
  gtk_box_pack_end(GTK_BOX(hboxmarker), dummy, FALSE, FALSE, 0);
  gtk_widget_show(dummy);
  gtk_widget_show(space1);
  gtk_widget_show(marker1);
  gtk_widget_show(space2);
  gtk_widget_show(marker2);
  animation.playposbox=playposbox=gtk_entry_new_with_max_length(5);
  gtk_widget_set_usize(playposbox,60,0);
  gtk_entry_set_text(GTK_ENTRY(playposbox), "0");
  gtk_box_pack_end(GTK_BOX(sliderhbox), playposbox, FALSE, FALSE, 5);

  gtk_widget_show(hboxmarker);
  gtk_widget_show(markereventbox);
  gtk_widget_show(playvbox);
  gtk_widget_show(playposbox);
  gtk_widget_show(playposslider);
  gtk_widget_show(sliderhbox);
  /* Connect these boxes */
  gtk_signal_connect (GTK_OBJECT (playposslideradj), "value_changed",
	    GTK_SIGNAL_FUNC (PlayPosSlider_Changed), playposbox);
  gtk_signal_connect (GTK_OBJECT (playposbox), "activate",
	    GTK_SIGNAL_FUNC (PlayPosBox_Changed), playposslideradj);

  /* play buttons */
  animation.playframe=frame;
  gtk_box_pack_start(GTK_BOX(mvbox),playhbox,FALSE,FALSE,5);
  tobeg=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMtobeg);
  rew=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMrew);
  play=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMplay);
  bstep=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMbstep);
  pause=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMpause);
  stop=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMstop);
  ff=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMff);
  toend=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMtoend);
  dummy=gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(playhbox),dummy,FALSE,FALSE,3);
  gtk_widget_show(dummy);
  playdir=Pixmap_Button_New_To_Hbox(frame,playhbox,&animation.playdirw,	
						playdirXPM[animation.playdir]);
  dummy=gtk_label_new("Frameskip");
  gtk_box_pack_start(GTK_BOX(playhbox),dummy,FALSE,FALSE,3);
  gtk_widget_show(dummy);
  frameskip=gtk_entry_new_with_max_length(4);
  gtk_widget_set_usize(frameskip,35,0);
  sprintf(s,"%d",anim.frameskip&8191);
  gtk_entry_set_text(GTK_ENTRY(frameskip), s);
  gtk_box_pack_start(GTK_BOX(playhbox), frameskip, FALSE, FALSE, 0);
  gtk_widget_show(frameskip);
  dummy=gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(playhbox),dummy,FALSE,FALSE,3);
  gtk_widget_show(dummy);
  redraw=Pixmap_Button_New_To_Hbox(frame,playhbox,&dummy,XPMredraw);
  animation.drawing=gtk_label_new("Wait");
  gtk_box_pack_start(GTK_BOX(playhbox),animation.drawing,FALSE,FALSE,3);

  /* signals */
  gtk_signal_connect(GTK_OBJECT (play),"clicked",
      	GTK_SIGNAL_FUNC(Menu_Run),(gpointer)0);
  gtk_signal_connect(GTK_OBJECT (stop),"clicked",
      	GTK_SIGNAL_FUNC(Menu_Run),(gpointer)1);
  gtk_signal_connect(GTK_OBJECT (pause),"clicked",
      	GTK_SIGNAL_FUNC(Pause_Signal),(gpointer)0);
  gtk_signal_connect(GTK_OBJECT (bstep),"clicked",
      	GTK_SIGNAL_FUNC(Pause_Signal),(gpointer)1);
  gtk_signal_connect(GTK_OBJECT (tobeg),"clicked",
      	GTK_SIGNAL_FUNC(BegEnd_Signal),(gpointer)0);
  gtk_signal_connect(GTK_OBJECT (toend),"clicked",
      	GTK_SIGNAL_FUNC(BegEnd_Signal),(gpointer)1);
  gtk_signal_connect(GTK_OBJECT (ff),"pressed",
      	GTK_SIGNAL_FUNC(FF_Signal),(gpointer)0);
  gtk_signal_connect(GTK_OBJECT (ff),"released",
      	GTK_SIGNAL_FUNC(FF_Signal),(gpointer)2);
  gtk_signal_connect(GTK_OBJECT (rew),"pressed",
      	GTK_SIGNAL_FUNC(FF_Signal),(gpointer)1);
  gtk_signal_connect(GTK_OBJECT (rew),"released",
      	GTK_SIGNAL_FUNC(FF_Signal),(gpointer)3);
  gtk_signal_connect(GTK_OBJECT (playdir),"clicked",
      	GTK_SIGNAL_FUNC(RotatePlayDir_Signal),NULL);
  gtk_signal_connect (GTK_OBJECT (frameskip), "activate",
	    GTK_SIGNAL_FUNC (Frameskip_Set), frameskip);
  gtk_signal_connect(GTK_OBJECT (redraw),"clicked",
      	GTK_SIGNAL_FUNC(Replot_Objects),(gpointer)1);

  gtk_widget_show(playhbox);
  gtk_widget_show(mvbox);

  /* Do not know how to put reshape event on a frame */
  gtk_signal_connect (GTK_OBJECT (playposslider), "size_allocate",
		      GTK_SIGNAL_FUNC (Reshape_Anim), NULL);
  gtk_widget_add_events(markereventbox,GDK_BUTTON_PRESS_MASK);
  gtk_widget_add_events(markereventbox,GDK_BUTTON_PRESS_MASK|
				      GDK_BUTTON1_MOTION_MASK|
				      GDK_BUTTON2_MOTION_MASK|
				      GDK_BUTTON3_MOTION_MASK);
  gtk_signal_connect (GTK_OBJECT (markereventbox), "button_press_event",
		      GTK_SIGNAL_FUNC (Marker_Move), NULL);
  gtk_signal_connect (GTK_OBJECT (markereventbox), "motion_notify_event",
		      GTK_SIGNAL_FUNC (Marker_Move), NULL);
}

void reset_anim()
{
  anim.framedt=10;
  animation.framedtlimitfail=0;
  anim.timeouttag=0;
  anim.running=0;
  anim.frameskip=1;
  animation.framepc=0;
  animation.playdir=0;
  anim.ff=1;
  animation.ffval=5;
  anim.adjustframedt=0;
  anim.adjustquadres=0;
  animation.quadresmin=4;
  animation.quadadjustdtlim=200;
  animation.donotplot=0;
  animation.framerepeat=1;
  animation.framerepeatcounter=0;
  animation.r=0.8;
  animation.g=0.6;
  animation.b=1.0;
  animation.a=0.8;
  animation.marker1=0;
  animation.marker2=-1;
  animation.xyzf=3000.0;
  anim.keyf=5.0;
  /* pre readrc reset */
  gl.width=gl.height=300;
  gl.bg[0]=0.0; gl.bg[1]=0.0;
  gl.bg[2]=0.0; gl.bg[3]=0.0;
  gl.f=300.0;
  gl.x=0.0; gl.y=0.0; gl.z=-3.0;
  gl.phix=0.0; gl.phiy=0.0; gl.phiz=0.0;
  gl.quad.st=8;
}
