/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/06/16 17:05:43 $ $Revision: 1.6 $ */
/* main */
#include <GL/osmesa.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "d2mglobal.h"
#ifndef _WIN32
#include <unistd.h>
#endif
#include <getopt.h>
#ifdef _WIN32
#include <direct.h>
#include <io.h>
#endif

#ifdef _WIN32
/*'conversion' conversion from 'type1' to 'type2', possible loss of data*/
#pragma warning(disable:4244)
#endif

/* Prototypes */
void io_setfiletype(int);
void reset_io(void);
void Read_Params(FILE *,char *);
void reset_gl(void);
void reset_objects(void);
void reset_BodyDisplaySettings(void);
void Write_os_Png(char *,void *,int,int);
void Movie(void);

void perr(char *format, ...)
{
  va_list arglist;

  va_start(arglist,format);
  vfprintf(stderr,format,arglist);
  va_end(arglist);
  exit(0);
}

int ilog(int f)
{
  int a;
  a=(int)(log10(fabs((double)f)+(f==0)));
  if (a<0) a=2-a;
  else a++;
  if (f<0) a++;
  return(a);
}

void reset_global(void)
{
  double deflight[4][4] =	{{ 0.0, 0.0, 1.0, 0.0 },
				 { 1.0, 1.0, 1.0, 0.0 },
				 { 1.0, 1.0, 1.0, 1.0 },
				 { 0.0, 0.0, 0.0, 0.0 }};
  int a,b;

  gl.filetype = 0;
#ifndef _WIN32
  gl.tmpdir = strdup("/tmp");
#else
  gl.tmpdir = strdup("./tmp");
#endif
  gl.outfile = strdup("movie.mpg");
  gl.pattern = strdup("IBBPBBPBBPBBPBBP");
  gl.pattlen = strlen(gl.pattern);
  gl.framerate = 25.0;
  gl.framerepeat = 1;
  gl.quadres = 32;
  gl.quality = 5;
  gl.x = 0.0;
  gl.y = -5.0;
  gl.z = -0.5;
  gl.f = 300.0;
  gl.phix = 0.0;
  gl.phiy = 0.0;
  gl.phiz = 0.0;
  gl.dx = 0.0;
  gl.dy = 0.0;
  gl.dz = 0.0;
  gl.dphix = 0.0;
  gl.dphiy = 0.0;
  gl.dphiz = 0.0;
  gl.startframe = 0;
  gl.endframe = 1;
  gl.frameskip = 1;
  gl.width = 300;
  gl.height = 300;
  gl.playdir = 0;
  gl.lightnum = 1;
  gl.light = (struct lightstr *) calloc(sizeof(struct lightstr),1);
  for (a=0;a<4;a++) { 
    gl.bg[a]=0.0;
    for (b=0;b<4;b++) { 
      gl.light->colors[a][b]=deflight[a][b];
    }
  }
}

void RemoveTmpDir(void)
{
  int a;

  if (gl.remove) {
    for (a=0;a<gl.i;a++) {
      sprintf(gl.png_name,gl.png_templ,a);
      remove(gl.png_name);
    }
    rmdir(gl.tmpdir);
  }
}

void Create_Tmp_Dir()
{
  int l;
  char *tmpdir;
  char *tmpdirname;

  tmpdir=(char *)calloc(strlen(gl.tmpdir)+15,sizeof(char));
  sprintf(tmpdir,"%s/movieXXXXXX",gl.tmpdir);
  if (tmpdir==NULL) perr("Unable to allocate memory for tmpname\n");
  free(gl.tmpdir);
#ifndef _WIN32
  tmpdirname=mkdtemp(tmpdir);
#else
  tmpdirname=mktemp(tmpdir);
  if((tmpdirname!=NULL)&&(mkdir(tmpdirname)!=0))
  {
    free(tmpdirname);
    tmpdirname=NULL;
  }
#endif
  if (tmpdirname==NULL) perr("Unable to create tmp dir%s\n",tmpdir);
  gl.tmpdir=tmpdirname;
  atexit(RemoveTmpDir);
  l=ilog(max(gl.startframe,gl.endframe))+strlen(tmpdirname)+10;
  gl.png_name=(char *)calloc(l,sizeof(char));
  gl.png_templ=(char *)calloc(l,sizeof(char));
  if (gl.png_templ==NULL) perr("Unable to allocate memory for tmpname\n");
  sprintf(gl.png_templ,"%s/%%0%dd.png",tmpdirname,
      			ilog(max(gl.startframe,gl.endframe)));
}

void Usage(void)
{
  fprintf(stderr,"usage: dat2mpg [-b max bodies] -m -r [paramfile]\n");
  exit(0);
}

int main(int argc, char **argv) 
{  
  OSMesaContext os_context=NULL;
  void *os_buffer=NULL;
  FILE *in;
  int ch;

  reset_io();
  body_alloc=50000;
  gl.movie=1;gl.remove=1;
  while ((ch=getopt(argc,argv,"b:h-:mr")) != EOF) {
    switch (ch) {
      case 'b'  : /* body alloc */
		  body_alloc=atoi(optarg);
		  if (body_alloc<1) Usage();
		  break;
      case 'm'  : /* do not make movie */
		  gl.movie=0;
		  break;
      case 'r'  : /* do not remove images */
		  gl.remove=0;
		  break;
      case '-'  : /* Anything with -- displays useage */
      case 'h'  : /* help */
      case '?'  :
      default   : Usage();
      		  break;
    }
  }
  if (optind<argc) {	/* filename */
    in=fopen(argv[optind],"r");
  } else {
    in=stdin;
  }

  reset_global();

  io_setfiletype(0);
  Read_Params(in,argv[0]);
  if (in!=stdin) fclose(in);
  gl.quad.st=gl.quadres;
  gl.quad.sl=gl.quadres*2;

  /* create an mesa context for gl */
  os_context = OSMesaCreateContext( OSMESA_RGB, NULL );
  if (os_context == NULL) perr("unable to create os context\n");
  gl.os_buffer=os_buffer = (void*) malloc( 3 * gl.width * gl.height );
  if (os_buffer==NULL) perr("os buffer allocation error\n");
  if (! OSMesaMakeCurrent( os_context, os_buffer, GL_UNSIGNED_BYTE,
		  gl.width, gl.height)) perr("os buffer bind error\n");

  reset_gl();
  reset_objects();
  reset_BodyDisplaySettings();
  if (gl.startframe!=gl.endframe) Create_Tmp_Dir();

  Movie();

  return(1);
}
