/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/11/03 17:36:59 $ $Revision: 1.38 $ */
#include <stdio.h>
#include "rpn.h"
#include "body.h"

#ifdef _WIN32
/*'conversion' conversion from 'type1' to 'type2', possible loss of data*/
#pragma warning(disable:4244)
#endif

#ifdef _MOZI
int ilog(double);
#else
int ilog(int);
#define animation anim
#endif
void perr(char *, ...);
void io_setfiletype(int);
int io_detfiletype(void);
int move_bp(char *,int);
double RPN_Evaluate(RPN *,double *);

/* These settings are valid for all objects */
void Set_General(BODY *body)
{
  int bo;

  glEnable(GL_COLOR_MATERIAL);
  bo=body->obj;
  glColor4f(bg.param[bo].color[0]*bg.cc[0],bg.param[bo].color[1]*bg.cc[1],
	bg.param[bo].color[2]*bg.cc[2],bg.param[bo].color[3]*bg.cc[3]);
  if (bg.param[bo].texture>0) {
    glBindTexture(GL_TEXTURE_2D,gl.texture.texture[bg.param[bo].texture-1]);
    glEnable(GL_TEXTURE_2D);
  } else {
    glDisable(GL_TEXTURE_2D);
  }

  if (bg.param[bo].blend) {
    glEnable(GL_BLEND);
  } else glDisable(GL_BLEND);

  glLoadIdentity();
  glTranslatef(gl.x,gl.y,gl.z);
  glRotatef(gl.phiy,0.0,1.0,0.0);
  glRotatef(gl.phix,1.0,0.0,0.0);
  glRotatef(gl.phiz,0.0,0.0,1.0);
  glRotatef(-90.0,1.0,0.0,0.0);
  glTranslatef(gl.f*body->x, gl.f*body->y,gl.f*body->z);
  glRotatef(body->phi*180/M_PI,body->rx,body->ry,body->rz);
}

int Constraint(int a)
{
  int b;
  double val;
  CONSTRAINT *ctmp;

  bg.cc[0]=1.0; bg.cc[1]=1.0; bg.cc[2]=1.0; bg.cc[3]=1.0;
  bg.fdx=1.0;bg.arrowdir=0;
  ctmp=bg.param[body[a].obj].constr;
  for (b=0;b<bg.param[body[a].obj].constr_num;b++) {
    if (ctmp!=NULL) {
      if (ctmp->str==NULL) {
	switch (ctmp->rel) {
	  case 0:	/* = */
	    if ((double)body[a].params[ctmp->param]!=ctmp->val) return(1);
	    break;
	  case 1:	/* > */
	    if ((double)body[a].params[ctmp->param]<=ctmp->val) return(1);
	    break;
	  case 2:	/* < */
	    if ((double)body[a].params[ctmp->param]>=ctmp->val) return(1);
	    break;
	  case 3: /* R */
	    bg.cc[0]*=body[a].params[ctmp->param]*ctmp->val;
	    break;
	  case 4: /* G */
	    bg.cc[1]*=body[a].params[ctmp->param]*ctmp->val;
	    break;
	  case 5: /* B */
	    bg.cc[2]*=body[a].params[ctmp->param]*ctmp->val;
	    break;
	}
      } else {
        /* RPN constraint */
	val=RPN_Evaluate(ctmp->rpnroot,body[a].params);
	switch (ctmp->strrel) {
	  case 0:	/* expression >0 */
	    if (val<=0.0) return(1);
	    break;
	  case 1:	/* R */
	    bg.cc[0]*=val;
	    break;
	  case 2:	/* G */
	    bg.cc[1]*=val;
	    break;
	  case 3:	/* B */
	    bg.cc[2]*=val;
	    break;
	  case 4:	/* RGB */
	    bg.cc[0]*=val;
	    bg.cc[1]*=val;
	    bg.cc[2]*=val;
	    break;
	  case 5:	/* Arrow */
	    bg.arrowdir=ctmp->param/CEF_SUBPARNUM;
	    if ((bg.arrowdir>1) && (bg.arrowdir!=4)) {
	      bg.arrowsize=fabs(val);
	      bg.arrowwidth=fabs(ctmp->val);
	    } else {
	      bg.arrowdir=0;
	    }
	    break;
	  case 6:	/* Size */
	    bg.fdx=val;
	    break;
	  case 7:	/* Cut */
	    if (fabs(-body[a].x*sin(gl.phiz/180.0*M_PI)+
	      body[a].y*cos(gl.phix/180.0*M_PI)*cos(gl.phiz/180.0*M_PI)+
	      body[a].z*sin(gl.phix/180.0*M_PI)*cos(gl.phiz/180.0*M_PI)+val)
	      	>fabs(ctmp->val)) return(1);
	    break;
	}
      }
      ctmp=ctmp->next;
    }
  }
  return(0);
}

void Plot_Body(int a,int mode)
{
  double r,q;
  double nx,ny,nz,tmp,phi;

  if (!bg.param[body[a].obj].show) return;
  if ((mode) && (!bg.param[body[a].obj].blend)) return;
  if ((!mode) && (bg.param[body[a].obj].blend)) return;
  if (Constraint(a)) return;
  Set_General(&body[a]);
  if (bg.arrowdir) {
    r=bg.param[body[a].obj].fcalscale*bg.fdx*gl.f;
  } else {
    if (bg.fdx==1.0) {
      r=body[a].sx*0.5*gl.f;		/* radius or linear size */
    } else {
      r=gl.f*bg.fdx;			/* radius or linear size */
    }
    r*=bg.param[body[a].obj].fcalscale;
  }
  switch (body[a].type+100*bg.arrowdir) {
    case BT_Wall:
      q=body[a].sy*0.5*gl.f;
      glBegin(GL_QUADS);
      glNormal3f(body[a].nx,body[a].nz,body[a].ny);
      glTexCoord2f(0.0, 1.0);
      glVertex3f(-r, q, 0.0);
      glTexCoord2f(0.0, 0.0);
      glVertex3f(-r,-q, 0.0);
      glTexCoord2f(1.0, 0.0);
      glVertex3f( r,-q, 0.0);
      glTexCoord2f(1.0, 1.0);
      glVertex3f( r, q, 0.0);
      glEnd();
      break;
    case BT_DGBall:
    case BT_STBall:
    case BT_BallCC:
    case BT_Ball:
      gluSphere(gl.quad.q, r, gl.quad.sl,gl.quad.st);
      break;
    case BT_Force:
    case BT_DTEdge:
    case BT_Cylinder:
      gluCylinder(gl.quad.q,r,r,body[a].sz*gl.f,gl.quad.sl,gl.quad.st);
      break;
    case BT_Disk:
      gluDisk(gl.quad.q, 0.0, r, gl.quad.sl,gl.quad.st);
      break;
    default:	/* Arrow */
      glRotatef(-body[a].phi*180/M_PI,body[a].rx,body[a].ry,body[a].rz);
      /* direction */
      nx=body[a].params[bg.arrowdir*CEF_SUBPARNUM];
      ny=body[a].params[bg.arrowdir*CEF_SUBPARNUM+1];
      nz=body[a].params[bg.arrowdir*CEF_SUBPARNUM+2];
      r=sqrt(nx*nx+ny*ny+nz*nz);
      nx/=r;ny/=r;nz/=r;
      phi=asin(sqrt(nx*nx+ny*ny))/M_PI*180.0;
      if (nz<0.0) glRotatef(180.0,1.0,0.0,0.0);
      glRotatef(-phi,ny*fsgn(nz),-nx,0.0);
      tmp=bg.arrowwidth*gl.f;
      gluCylinder(gl.quad.q,tmp,0.0,bg.arrowsize*gl.f,gl.quad.sl,gl.quad.st);
      glTranslatef(0.0,0.0,bg.arrowsize*gl.f*0.7);
      gluCylinder(gl.quad.q,2.0*tmp,0.0,bg.arrowsize*gl.f*0.3,
      					gl.quad.sl,gl.quad.st);
  }
}

void Plot_Objects(void)
{
  int a,ob;

  /* scale force */

  for (a=0;a<bg.obj_num;a++) bg.param[a].fcalscale=0.0;
  for (a=0;a<bg.body_num;a++) {
    ob=body[a].obj;
    if (bg.param[ob].scale_force) {
      if (body[a].sx>bg.param[ob].fcalscale) {
	bg.param[ob].fcalscale=body[a].sx;
      }
    }
  }
  for (a=0;a<bg.obj_num;a++) {
    if (bg.param[a].fcalscale>0.0) {
      bg.param[a].fcalscale=bg.param[a].fsize/bg.param[a].fcalscale;
    } else bg.param[a].fcalscale=bg.param[a].fscale;
  }
  
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
  glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
  for (a=0;a<bg.body_num;a++) Plot_Body(a,0);
  glFlush();
#ifdef _MOZI
  if (ll.moveit) {	/* if light is moved then plot the sun(s) */
    GLfloat suncolor[4]={1.0,1.0,0.0,1.0};

    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,suncolor);
    glEnable(GL_COLOR_MATERIAL);
    glColor4f(1.0,1.0,0.0,1.0);
    glDisable(GL_TEXTURE_2D);
    for (a=0;a<MaxLight;a++) {
      if (gl.light[a].on) {
	glLoadIdentity();
	glTranslatef(gl.x, gl.z,gl.y);
	glTranslatef(gl.light[a].param[0][0],gl.light[a].param[0][1],
	    		gl.light[a].param[0][2]);
	gluSphere(gl.quad.q, 0.1, gl.quad.sl,gl.quad.st);
      }
    }
  }
#endif
  for (a=0;a<bg.body_num;a++) Plot_Body(a,1);
  glFlush();
}

void Read_Objects(int pos)
{
  int a,n,d,i,ll,bs,on;
  int typ,tex,num,dummyint;
  double x,y,z,rx,ry,rz,phi;
  char *p;

  ll=bg.line_length;
  if (bg.line_length<=0) {
    io.seek(io.in,animation.framepos[pos],SEEK_SET);
    if ((bg.input_type==4) || (bg.input_type==2)) {
      ll=animation.framepos[pos+1]-animation.framepos[pos];
      a=io.read(io.in,ll);
      if (a!=ll) {
	fprintf(stderr,"Mozi warning: in data file type %d at",bg.input_type);
	fprintf(stderr," pos %d short line (%d/%d)\n",pos,a,bg.line_length);
	return;
      }
    } else {
      /* text input */
      io.getline(io.in);
    }
    io.bp=move_bp(io.buff,1);
  } else {
    /* we have fixed line length */
    io.seek(io.in,animation.framepos[0]+pos*bg.line_length,SEEK_SET);
    a=io.read(io.in,bg.line_length);
    if (a!=bg.line_length) {
      fprintf(stderr,"Mozi warning: in data file type %d at",bg.input_type);
      fprintf(stderr," pos %d short line (%d/%d)\n",pos,a,bg.line_length);
      return;
    }
  }
  bg.body_num=bg.wall_num;
  switch (bg.input_type) {
    case 4:
      i=sizeof(int);d=sizeof(double);
      p=io.buff+d;
      /* object type is byte, end if it is zero */
      while ((*p) && (p<ll+io.buff)) {
	typ=*((int *)p);p+=i;
	tex=*p++;
	num=*((int *)p);p+=i;
	bs=bg.body_num;
	bg.body_num+=max(1,num);
	if (bg.body_num>body_alloc) bg.body_num=body_alloc;
	for (a=bs;a<bg.body_num;a++) {
	  if (a>body_alloc) break;
	  body[a].texture=tex&15;
	  body[a].type=typ&BP_Mask;
	  body[a].par=typ;
	  if (num==0) break;
	  /* get size of body. It's compulsory */
	  body[a].sx=*((double *)p);p+=d;
	  if ((body[a].type==BT_Cylinder) || (body[a].type==BT_Force)
	      || (body[a].type==BT_DTEdge)) {
	    body[a].sz=*((double *)p);p+=d;
	  }
	  if (body[a].type==BT_BallCC) {
	    body[a].params[4*CEF_SUBPARNUM]=*((double *)p);p+=i;
	    body[a].sy=(double)body[a].params[4*CEF_SUBPARNUM];
	  }
	  if (body[a].type==BT_Wall) {
	    body[a].sy=*((double *)p);p+=d;
	  }
	  if ((body[a].type==BT_Force) || (body[a].type==BT_DTEdge)) {
	    body[a].ind1=*((int *)p);p+=i;
	    body[a].ind2=*((int *)p);p+=i;
	  }
	  /* position */
	  if (typ&BP_Pos) {
	    body[a].x=*((double *)p);p+=d;
	    body[a].y=*((double *)p);p+=d;
	    body[a].z=*((double *)p);p+=d;
	  } else body[a].x=body[a].y=body[a].z=0.0;
	  /* orientation */
	  if (typ&BP_Orient) {
	    body[a].rx=*((double *)p);p+=d;
	    body[a].ry=*((double *)p);p+=d;
	    body[a].rz=*((double *)p);p+=d;
	    body[a].phi=*((double *)p);p+=d;
	  } else {
	    body[a].rx=body[a].ry=body[a].phi=0.0;
	    body[a].rz=1.0;
	  }
	  /* other parameters */
	  on=4*CEF_SUBPARNUM;
	  if (typ&BP_Coord) {
	    bs=*((int *)p);
	    body[a].params[on]=(double)bs;
	    p+=i;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_Velo) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_Force) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_AngVelo) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_Torque) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_TrVelo) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_TrForce) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  on+=CEF_SUBPARNUM;
	  if (typ&BP_TrAngVelo) {
	    body[a].params[on]=*((double *)p);p+=d;
	    body[a].params[on+1]=*((double *)p);p+=d;
	    body[a].params[on+2]=*((double *)p);p+=d;
	  }
	  if (tex&16) body[a].appearance ^= BA_Texture2D;
	  if (tex&32) body[a].appearance |= BA_Blend;
	}
      }
      break;
    case 3:
      perr("data file type 3 is not supported in mozi\n");
      break;
    case 2:
      i=sizeof(int);d=sizeof(double);
      p=io.buff+d;
      /* object type is byte, end if it is zero */
      while ((*p) && (p<bg.line_length+io.buff)) {
	typ=*p++;
	tex=*p++;
	num=*((int *)p);p+=i;
	for (a=bg.body_num;a<bg.body_num+num;a++) {
	  if (a>body_alloc) break;
	  body[a].type=typ;
	  body[a].x=*((double *)p);p+=d;
	  body[a].y=*((double *)p);p+=d;
	  body[a].z=*((double *)p);p+=d;
	  body[a].rx=*((double *)p);p+=d;
	  body[a].ry=*((double *)p);p+=d;
	  body[a].rz=*((double *)p);p+=d;
	  body[a].phi=*((double *)p);p+=d;
	  body[a].sx=*((double *)p);p+=d;
	  if (body[a].type==BT_Cylinder) {
	    body[a].sz=*((double *)p);p+=d;
	  }
	  if (body[a].type==BT_BallCC) p+=i;
	  if (body[a].type==BT_Wall) {
	    body[a].sy=*((double *)p);p+=d;
	  }
	  body[a].texture=tex&15;
	  body[a].appearance=BA_Color+BA_Texture2D;
	  if (tex&16) body[a].appearance ^= BA_Texture2D;
	  if (tex&32) body[a].appearance |= BA_Blend;
	}
	bg.body_num+=num;
      }
      if (bg.body_num>body_alloc) bg.body_num=body_alloc;
      break;
    case 1:
      while ((io.scan("%d",&typ)==1)  && (bg.body_num<body_alloc)) {
	io.scan("%d",&tex); io.scan("%d",&num);
	for (a=bg.body_num;a<bg.body_num+num;a++) {
	  if (a>body_alloc) break;
	  body[a].type=typ;
	  io.scan("%le",&body[a].x);
	  io.scan("%le",&body[a].y);
	  io.scan("%le",&body[a].z);
	  io.scan("%le",&body[a].rx);
	  io.scan("%le",&body[a].ry);
	  io.scan("%le",&body[a].rz);
	  io.scan("%le",&body[a].phi);
	  io.scan("%le",&body[a].sx);
	  if (body[a].type==BT_Cylinder) io.scan("%le",&body[a].sz);
	  if (body[a].type==BT_BallCC) io.scan("%d",&dummyint);
	  if (body[a].type==BT_Wall) io.scan("%le",&body[a].sy);
	  body[a].texture=tex&15;
	  body[a].appearance=BA_Color+BA_Texture2D;
	  if (tex&16) body[a].appearance ^= BA_Texture2D;
	  if (tex&32) body[a].appearance |= BA_Blend;
	}
	bg.body_num+=num;
      }
      if (bg.body_num>body_alloc) bg.body_num=body_alloc;
      break;
    case 0:
    default:
      io.scan("%d",&n);	/* read the number of balls */
      bg.body_num=bg.wall_num+n;
      bg.body_num=min(bg.body_num,body_alloc);
      for (a=bg.wall_num;a<bg.body_num;a++) {
	io.scan("%le",&x);io.scan("%le",&y);io.scan("%le",&z);
	io.scan("%le",&rx);io.scan("%le",&ry);io.scan("%le",&rz);
	io.scan("%le",&phi);
	body[a].x=x;
	body[a].y=y;
	body[a].z=z;
	body[a].rx=rx;
	body[a].ry=ry;
	body[a].rz=rz;
	body[a].phi=phi;
      }
      break;
  }
  if (bg.obj_num) {
    bs=0;
    for (a=0;a<bg.body_num;a++) {
      while (body[a].type!=bg.param[bs].type) bs++;
      body[a].obj=bs;
      body[a].params[0]=(double)a;
      on=CEF_SUBPARNUM;
      body[a].params[on]=body[a].sx;
      body[a].params[on+1]=body[a].sy;
      body[a].params[on+2]=body[a].sz;
      body[a].params[on+3]=(double)body[a].ind1;
      body[a].params[on+4]=(double)body[a].ind2;
      on+=CEF_SUBPARNUM;
      body[a].params[on]=body[a].x;
      body[a].params[on+1]=body[a].y;
      body[a].params[on+2]=body[a].z;
      on+=CEF_SUBPARNUM;
      body[a].params[on]=body[a].rx;
      body[a].params[on+1]=body[a].ry;
      body[a].params[on+2]=body[a].rz;
      body[a].params[on+3]=body[a].phi;
    }
  }
}

void Not_Fixed_Linelength()
{
  int i,d,d3,v,llmax,typ,num,lpos;
  int bases[10];
  long *buff;

  i=sizeof(int);d=sizeof(double);d3=3*d;
  bases[BT_EOL]=0;bases[BT_Wall]=d+d;bases[BT_Ball]=d;
  bases[BT_Cylinder]=d+d;bases[BT_Disk]=d;bases[BT_BallCC]=d+i;
  bases[BT_Force]=d+d+i+i;bases[BT_STBall]=d;bases[BT_DGBall]=d;
  bases[BT_DTEdge]=d+d+i+i;
  llmax=0;lpos=io.tell(io.in);
  do {
    if (llmax<io.tell(io.in)-lpos) llmax=io.tell(io.in)-lpos;
    animation.framepos[animation.maxframe++]=io.tell(io.in);
    if (animation.maxframe==animation.framepossize) {
      /* buffer is full */
      buff=animation.framepos;
      animation.framepos=(long *)
		      calloc(animation.framepossize*2,sizeof(long));
      memcpy(animation.framepos,buff,animation.framepossize*sizeof(long));
      free(buff);
      animation.framepossize*=2;
    }
    io.seek(io.in,d,SEEK_CUR);
    do {
      io.read(io.in,i);
      typ=*((int *)io.buff);
      if (typ) {
	io.seek(io.in,1,SEEK_CUR);
	io.read(io.in,i);
	num=*((int *)io.buff);
	v=bases[typ&BP_Mask];
	if (typ&BP_Pos) v+=d3;
	if (typ&BP_Orient) v+=d3+d;
	if (typ&BP_Coord) v+=i;
	if (typ&BP_Velo) v+=d3;
	if (typ&BP_Force) v+=d3;
	if (typ&BP_AngVelo) v+=d3;
	if (typ&BP_Torque) v+=d3;
	if (typ&BP_TrVelo) v+=d3;
	if (typ&BP_TrForce) v+=d3;
	if (typ&BP_TrAngVelo) v+=d3;
	io.seek(io.in,v*num,SEEK_CUR);
      }
    } while ((!io.eof(io.in)) && (typ));
  } while (!io.eof(io.in));
  if (llmax<io.tell(io.in)-lpos) llmax=io.tell(io.in)-lpos;
  animation.maxframe--;
}

void reset_objects()
{
  int a,tex,typ,p,num,size_int,size_dble,goon;
  long pos,*buff;
  char ch;

  bg.line_length=-1;
  if (!io_detfiletype()) perr("input file %s does not exist\n",io.filename);
  io.in=io.open(io.filename,"rb");
  if (io.in==NULL) perr("Input file %s not found\n",io.filename);

  body=(BODY *)calloc(body_alloc,sizeof(BODY));
  for (a=0;a<body_alloc;a++) {
    body[a].type=BT_Ball;
    body[a].x=0.0;
    body[a].y=0.0;
    body[a].z=0.0;
    body[a].nx=0.0;
    body[a].ny=0.0;
    body[a].nz=-1.0;
    body[a].rx=0.0;
    body[a].ry=0.0;
    body[a].rz=1.0;
    body[a].phi=0.0;
    body[a].sx=0.001;
    body[a].sy=0.0;
    body[a].sz=0.0;
    body[a].appearance=BA_Color+BA_Texture2D;
    body[a].texture=1;
    body[a].material=GL_DIFFUSE;
    body[a].par=96;
  }
  bg.wall_num=0;

  /* detect frame positions */
  animation.framepossize=1000;
  animation.framepos=(long *) calloc(animation.framepossize,sizeof(long));
  animation.maxframe=0;
  animation.framepc=0;
  bg.input_type=0;goon=1;
  do {
    pos=io.tell(io.in);
    io.getline(io.in);
    ch=io.buff[0];
    /* skip comments */
    if ((!io.eof(io.in)) && (ch!='#')) {
      animation.framepos[animation.maxframe++]=pos;
      if (animation.maxframe==animation.framepossize) {
	/* buffer is full */
	buff=animation.framepos;
	animation.framepos=(long *)
	  		calloc(animation.framepossize*2,sizeof(long));
	memcpy(animation.framepos,buff,animation.framepossize*sizeof(long));
	free(buff);
	animation.framepossize*=2;
      }
    } else if (ch=='#') {	/* special */
      if (strlen(io.buff)==2) {	/* A single # character in a line */
	if (bg.line_length>0) {
	  /* We have the position information in the header so we quit */
	  animation.framepos[0]=pos=io.tell(io.in);
	  animation.framepossize=
	  	animation.maxframe=(io.tell(io.in)-pos)/bg.line_length;
	} else {
	  Not_Fixed_Linelength();
	}
	goon=0;
	break;
      }
      if (strncmp(io.buff+1,"object",6)==0) {	/* indeed */
	p=1+strlen("object");
	sscanf(io.buff+p,"%d %d %d",&typ,&tex,&num);
	p+=move_bp(io.buff+p,3);
	for (a=bg.wall_num;a<bg.wall_num+num;a++) {
	  body[a].type=typ;
	  sscanf(io.buff+p,"%lf %lf %lf %lf %lf %lf %lf %lf",
	      &body[a].x,&body[a].y,&body[a].z,&body[a].rx,
	      &body[a].ry,&body[a].rz,&body[a].phi,&body[a].sx);
	  p+=move_bp(io.buff+p,8);
	  if ((body[a].type==BT_Cylinder) || (body[a].type==BT_BallCC)) {
	    sscanf(io.buff+p,"%lf",&body[a].sz);
	    p+=move_bp(io.buff+p,1);
	  }
	  if (body[a].type==BT_Wall) {
	    sscanf(io.buff+p,"%lf",&body[a].sy);
	  }
	  body[a].texture=tex&15;
	  body[a].appearance=BA_Color+BA_Texture2D;
	  if (tex&16) body[a].appearance ^= BA_Texture2D;
	  if (tex&32) body[a].appearance |= BA_Blend;
	}
	bg.wall_num+=num;
      } else if (strncmp(io.buff+1,"type",4)==0) {	/* type is different */
	p=1+strlen("type");
	sscanf(io.buff+p,"%d",&bg.input_type);
	if (bg.input_type==3) {
	  perr("data file type 3 is not supported in mozi\n");
	}
	if ((bg.input_type==2) || (bg.input_type==4)) {
	  /* read size of int/dble, line length */
	  p+=move_bp(io.buff+p,1);
	  sscanf(io.buff+p,"%d %d %d",&size_int,&size_dble,&bg.line_length);
	  if ((size_int!=sizeof(int)) || (size_dble!=sizeof(double))) {
	    perr("Size of int or double does not match!\n");
	  }
	  if (io.linelen<bg.line_length) {
	    /* we reallocate io.buff */
	    free(io.buff);
	    io.linelen=bg.line_length+1;
	    io.buff=(char *)calloc(io.linelen,sizeof(char));
	  }
	}
      }
    }
  } while ((!io.eof(io.in)) && (goon));
  animation.maxframe--;
  bg.body_num=bg.wall_num;
  io.rewind(io.in);
#ifdef _MOZI
  if (animation.marker2<0) {
    animation.marker2=animation.maxframe;
  }
  bg.obj_num=0;
#endif

}

void reset_BodyDisplaySettings()
{
  int a,b,typ,tex;

  Read_Objects(0);	/* read objects we have */
  bg.obj_num=0;typ=tex=0;
  for (a=0;a<bg.body_num;a++) {
    if ((body[a].type!=typ) || (body[a].texture!=tex)) {
      bg.obj_num++;
      typ=body[a].type;tex=body[a].texture;
    }
  }
  /* allocate memory */
  bg.param=(bodydisplaystr *) calloc(bg.obj_num,sizeof(bodydisplaystr));
  b=typ=tex=0;
  for (a=0;a<bg.body_num;a++) {
    if ((body[a].type!=typ) || (body[a].texture!=tex)) {
      typ=body[a].type;tex=body[a].texture;
      bg.param[b].show=1;
      bg.param[b].type=typ;
      bg.param[b].par=body[a].par;
      bg.param[b].blend=body[a].appearance&BA_Blend;
      bg.param[b].texture=((body[a].appearance&BA_Texture2D) ? tex+1 : 0);
      bg.param[b].color[0]=0.8f;
      bg.param[b].color[1]=0.6f;
      bg.param[b].color[2]=1.0f;
      bg.param[b].color[3]=0.4f;
      bg.param[b].fscale=1.0;
      bg.param[b].fsize=0.001;
      bg.param[b].scale_force=0;
      bg.param[b].constr=NULL;
      bg.param[b].newconstr=(CONSTRAINT *) malloc(sizeof(CONSTRAINT));
      /* all objects */
      bg.param[b].newconstr->param=0;
      bg.param[b].newconstr->rel=1;
      bg.param[b].newconstr->val= -1.0;
      bg.param[b].newconstr->str=NULL;
      bg.param[b].newconstr->next=NULL;
      bg.param[b].newconstr->strrel=0;
      b++;
    }
    body[a].obj=b-1;
  }
}

/* EOF */
